local expanded={}
local nameFilter
local minLevelFilter, maxLevelFilter
local invSlotFilters={}
local numInvSlotFilters
local subClassFilters={}
local numSubClassFilters
local availableFilter

local function SaveFilter(stateTable, listFunction, getFunction)
	local numFilters=select("#", listFunction())
	for idx=0, numFilters do
		stateTable[idx]=getFunction(idx) and 1 or 0
	end
	return numFilters
end

local function RestoreFilter(stateTable, numFilters, setFunction)
	local firstExclusive=1
	for idx=0, numFilters do
		if(stateTable[idx]==1) then
			setFunction(idx, stateTable[idx], firstExclusive)
			-- 0 = all slots, we don't need to set any more after that.
			-- Setting more values even causes problems sometimes.
			-- For example subclass filter have all position set when "All" is selected, not only 0th,
			-- and resetting them that way make strange things happen with filters.
			if(idx==0) then return end
		end
		if(firstExclusive==1 and stateTable[idx]==1) then firstExclusive=0 end
	end
end

local function ExpandAllTradeskill()
	nameFilter=GetTradeSkillItemNameFilter()
	SetTradeSkillItemNameFilter(nil)

	minLevelFilter, maxLevelFilter=GetTradeSkillItemLevelFilter()
	SetTradeSkillItemLevelFilter(0, 0)
	if (IsAddOnLoaded("Jobber")) then
		availableFilter=JobberFrameAvailableFilterCheckButton:GetChecked()
	else
		availableFilter=TradeSkillFrameAvailableFilterCheckButton:GetChecked()
	end
	TradeSkillOnlyShowMakeable(0)

	numInvSlotFilters=SaveFilter(invSlotFilters, GetTradeSkillInvSlots, GetTradeSkillInvSlotFilter)
	numSubClassFilters=SaveFilter(subClassFilters, GetTradeSkillSubClasses, GetTradeSkillSubClassFilter)
	SetTradeSkillSubClassFilter(0, 1, 1)
	SetTradeSkillInvSlotFilter(0, 1, 1)

	for idx=GetNumTradeSkills(), 1, -1 do
		local skillName, skillType, numAvailable, isExpanded = GetTradeSkillInfo(idx)
		if(skillType == "header" and not isExpanded) then
			ExpandTradeSkillSubClass(idx)
			expanded[#expanded+1]=idx
		end
	end
end

local function RestoreAllTradeskill()
	for idx=#expanded, 1, -1 do
		CollapseTradeSkillSubClass(expanded[idx])
		expanded[idx]=nil
	end

	SetTradeSkillItemNameFilter(nameFilter)
	SetTradeSkillItemLevelFilter(minLevelFilter, maxLevelFilter)
	TradeSkillOnlyShowMakeable(availableFilter)

	RestoreFilter(subClassFilters, numSubClassFilters, SetTradeSkillSubClassFilter)
	RestoreFilter(invSlotFilters, numInvSlotFilters, SetTradeSkillInvSlotFilter)
end

TradeJunkie.ExpandAllTradeskill=ExpandAllTradeskill
TradeJunkie.RestoreAllTradeskill=RestoreAllTradeskill