local tmpStorage = {}
local function GetLearnSourceData(entry, suffix, ...)
	for idx=1, select("#", ...) do
		local data = entry[select(idx, ...)]
		if(data == nil) then return nil end
		tmpStorage[idx] = data
	end
	return unpack(tmpStorage)
end

local foundSources = {}
local foundSourcesNum = 0
local currentSourceNum
local factionSuffix

local function AddSource(format, ...)
	local number, color, faction, resetColor = "", "", "", ""
	if(currentSourceNum>1) then
		if(currentSourceNum==2) then foundSources[1] = "1. "..foundSources[1] end
		number = currentSourceNum ..". "
	end
	if(factionSuffix == opposingFactionSuffix) then
        color = "|cFF656565"
		faction = "("..'|cFFFF0000' .. opposingFactionText..color..") "
	elseif (factionSuffix == homeFactionSuffix) then
        color = "|cFFCCCCCC"
        faction = "("..'|cFF00FF00' .. homeFactionText..color..") "
    else
        color = "|cFFCCCCCC"
        if (format ~= fmtCrafted and format ~= fmtTrainerUnknown and format ~= fmtWorldDrop and format ~= fmtWorldDropUnknown and format ~= fmtMobDrop and format ~= fmtMobDropwc) then
            faction = "";
        end
    end
	foundSources[currentSourceNum] = color .. number..faction..string.format(format, ...)
	currentSourceNum = currentSourceNum + 1
end

local factionStandingCache={}
local function GetLearnFactionData(entry)
	local noCache
	local info = entry
	for idx,n in pairs(info["sources"]) do
		if(info["sources"][idx] == nil) then 
			break 
		else 
			entry = info["sources"][idx] 
		end
		if(entry["faction"]) then
			factionSuffix = entry["faction"]
		else
			factionSuffix = ""
		end
		
		suffix = factionSuffix
		
		local sourceType = entry["type"]
		if(not sourceType) then break end
        sourceType = strlower(sourceType);
		if(sourceType == "vendor") then
			local vendor, zone = GetLearnSourceData(entry, suffix, "vendor", "zone")
            local limited = GetLearnSourceData(entry ,"", "limited");
			if (not (vendor and zone)) then
				AddSource(fmtSourceVendorUnknown)
			elseif (not limited) then
                AddSource(fmtSourceVendor, vendor, zone)
            else
                AddSource(fmtSourceVendorLimited, vendor, zone)
            end
		elseif(sourceType == "faction") then
			local faction, standing, vendor, zone = GetLearnSourceData(entry, suffix, "faction", "standing", "vendor", "zone")
			if (faction and standing and vendor and zone) then
				local standingText = factionStandingCache[standing]
				if(not standingText) then
					standingText = GetText("FACTION_STANDING_LABEL"..standing, playerGender)
					factionStandingCache[standing] = standingText
				end
				local currentStanding = TradeJunkie.GetFactionStanding(faction)
				noCache = true
				if( currentStanding == nil or currentStanding < standing) then
					standingText = "|cFFFF0000"..standingText.."|r"
				end
				AddSource(fmtSourceFactionVendor, '|cFF00CCFF' .. faction .. '|r', standingText, vendor, zone)
			end
		elseif(sourceType == "drop") then 
			local mob, zone = GetLearnSourceData(entry, suffix, "mob", "zone")
			AddSource(fmtMobDrop, mob, zone)
        elseif(sourceType == "event") then
            local object,event = GetLearnSourceData(entry, suffix, "mob", "zone")
            if(object and event) then
                AddSource(fmtEventDrop, object, event)
            end
		elseif(sourceType == "worlddrop") then 
			local mobLevel = GetLearnSourceData(entry, suffix, "mobLevel")
			if(mobLevel) then
				AddSource(fmtWorldDrop, mobLevel)
			else
				AddSource(fmtWorldDropUnknown )
			end
		elseif(sourceType == "quest") then 
			local quest, giver, zone = GetLearnSourceData(entry, suffix, "quest", "giver", "zone")
			if (quest and giver and zone) then
				AddSource(fmtQuestRewardNC, quest, giver, zone)
			else
				AddSource(fmtQuestRewardUnknown)
			end
		elseif(sourceType == "discover") then
			AddSource(fmtDiscovery);
		elseif(sourceType == "trainer") then 
            AddSource(fmtTrainerUnknown)
        elseif(sourceType == "auction") then
            AddSource(fmtAuctionHouse);
        elseif(sourceType == "other") then
            local info = GetLearnSourceData(entry, suffix, "info")
            AddSource(info)
        elseif(sourceType == "crafted") then
            local info = GetLearnSourceData(entry, suffix, "crafted")
            AddSource(fmtCrafted,info)
        elseif(sourceType == "starter") then
        	   AddSource(fmtStarter);
		end
		idx = idx + 1
	end
	return noCache
end

local function GetLearnText(entry)
	local noCache
	local noCacheThis
	currentSourceNum = 1
	--factionSuffix = homeFactionSuffix noCacheThis = GetLearnFactionData(entry, homeFactionSuffix)
	--noCache = noCache or noCacheThis
	factionSuffix = "" noCacheThis = GetLearnFactionData(entry)
	noCache = noCache or noCacheThis
	--factionSuffix = opposingFactionSuffix noCacheThis = GetLearnFactionData(entry)
	--noCache = noCache or noCacheThis
	if(currentSourceNum == 1) then return nil end
	for idx = currentSourceNum, foundSourcesNum do
		foundSources[idx] = nil
	end
	foundSourcesNum = currentSourceNum - 1
	return table.concat(foundSources, "\n"), noCache
end

if(not TradeJunkie) then TradeJunkie={} end
TradeJunkie.GetLearnText=GetLearnText